package cn.wustlinghang.wusthelper.internal.graduate.services;

import cn.wustlinghang.mywust.core.parser.graduate.GraduateCourseTableParser;
import cn.wustlinghang.mywust.core.parser.graduate.GraduateScoreParser;
import cn.wustlinghang.mywust.core.parser.graduate.GraduateStudentInfoPageParser;
import cn.wustlinghang.mywust.core.parser.graduate.GraduateTrainingPlanPageParser;
import cn.wustlinghang.mywust.data.global.Course;
import cn.wustlinghang.mywust.data.global.Score;
import cn.wustlinghang.mywust.data.global.StudentInfo;
import cn.wustlinghang.mywust.data.undergrad.ExamDelayApplication;
import cn.wustlinghang.wusthelper.internal.graduate.exception.GraduateRpcException;
import cn.wustlinghang.wusthelper.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.util.List;

@ApplicationScoped
public class ParseService {

    private final GraduateCourseTableParser courseTableParser;
    private final GraduateScoreParser scoreParser;
    private final GraduateStudentInfoPageParser studentInfoPageParser;
    private final GraduateTrainingPlanPageParser trainingPlanPageParser;

    public ParseService(GraduateCourseTableParser courseTableParser,
                        GraduateScoreParser scoreParser,
                        GraduateStudentInfoPageParser studentInfoPageParser,
                        GraduateTrainingPlanPageParser trainingPlanPageParser) {

        this.courseTableParser = courseTableParser;
        this.scoreParser = scoreParser;
        this.studentInfoPageParser = studentInfoPageParser;
        this.trainingPlanPageParser = trainingPlanPageParser;
    }

    public List<Course> parseCourseTable(String data) throws GraduateRpcException {
        try {
            return courseTableParser.parse(data);
        } catch (Exception e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.PARSE_EXCEPTION,
                    GraduateRpcException.SubModuleCode.COURSE_TABLE,
                    GraduateRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public List<Score> parseScore(String data) throws RpcException {
        try {
            return scoreParser.parse(data);
        } catch (Exception e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.PARSE_EXCEPTION,
                    GraduateRpcException.SubModuleCode.SCORE,
                    GraduateRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public StudentInfo parseStudentInfo(String data) throws RpcException {
        try {
            return studentInfoPageParser.parse(data);
        } catch (Exception e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.PARSE_EXCEPTION,
                    GraduateRpcException.SubModuleCode.STUDENT_INFO,
                    GraduateRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public String parseTrainingPlan(String data) throws RpcException {
        try {
            return trainingPlanPageParser.parse(data);
        } catch (Exception e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.PARSE_EXCEPTION,
                    GraduateRpcException.SubModuleCode.TRAINING_PLAN,
                    GraduateRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
