package cn.wustlinghang.wusthelper.internal.graduate.services;

import cn.wustlinghang.mywust.core.request.service.auth.GraduateLogin;
import cn.wustlinghang.mywust.core.request.service.graduate.GraduateCourseTableApiService;
import cn.wustlinghang.mywust.core.request.service.graduate.GraduateScoreApiService;
import cn.wustlinghang.mywust.core.request.service.graduate.GraduateStudentInfoApiService;
import cn.wustlinghang.mywust.core.request.service.graduate.GraduateTrainingPlanApiService;
import cn.wustlinghang.mywust.core.request.service.undergraduate.*;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.graduate.exception.GraduateRpcException;
import cn.wustlinghang.wusthelper.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;

@Slf4j
@ApplicationScoped
public class RequestAgentService {

    private final RequestClientOption requestClientOption;

    private final GraduateCourseTableApiService courseTableApiService;
    private final GraduateScoreApiService scoreApiService;
    private final GraduateStudentInfoApiService studentInfoApiService;
    private final GraduateTrainingPlanApiService trainingPlanApiService;

    public RequestAgentService(RequestClientOption requestClientOption,
                               GraduateCourseTableApiService courseTableApiService,
                               GraduateScoreApiService scoreApiService,
                               GraduateStudentInfoApiService studentInfoApiService,
                               GraduateTrainingPlanApiService trainingPlanApiService) {

        this.requestClientOption = requestClientOption;

        this.courseTableApiService = courseTableApiService;
        this.scoreApiService = scoreApiService;
        this.studentInfoApiService = studentInfoApiService;
        this.trainingPlanApiService = trainingPlanApiService;
    }

    public String getStudentInfoPage(String cookie) throws RpcException {
        try {
            return studentInfoApiService.getPage(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, GraduateRpcException.SubModuleCode.STUDENT_INFO);
        } catch (IOException e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    GraduateRpcException.SubModuleCode.STUDENT_INFO,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String getCourseTable(String cookie) throws RpcException {
        try {
            return courseTableApiService.getPage(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, GraduateRpcException.SubModuleCode.COURSE_TABLE);
        } catch (IOException e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    GraduateRpcException.SubModuleCode.COURSE_TABLE,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String getScore(String cookie) throws RpcException {
        try {
            return scoreApiService.getPage(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, GraduateRpcException.SubModuleCode.SCORE);
        } catch (IOException e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    GraduateRpcException.SubModuleCode.SCORE,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String getTrainingPlan(String cookie) throws RpcException {
        try {
            return trainingPlanApiService.getPage(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, GraduateRpcException.SubModuleCode.TRAINING_PLAN);
        } catch (IOException e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    GraduateRpcException.SubModuleCode.TRAINING_PLAN,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    private GraduateRpcException wrapApiException(ApiException e, GraduateRpcException.SubModuleCode subModuleCode) {
        return switch (e.getCode()) {
            case NETWORK_EXCEPTION -> new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    subModuleCode,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
            case COOKIE_INVALID -> new GraduateRpcException(
                    GraduateRpcException.TypeCode.AUTH_EXCEPTION,
                    subModuleCode,
                    GraduateRpcException.ErrorCode.COOKIE_INVALID
            );
            default -> {
                log.error("研究生：{}代理请求异常，异常未处理", subModuleCode.name());
                log.error("异常：", e);
                yield new GraduateRpcException(
                        GraduateRpcException.TypeCode.AUTH_EXCEPTION,
                        subModuleCode,
                        GraduateRpcException.ErrorCode.COOKIE_INVALID
                );
            }
        };
    }
}
