package cn.wustlinghang.wusthelper.internal.library.services;

import cn.wustlinghang.mywust.core.request.service.library.LibraryApiService;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.library.exception.LibraryRpcException;
import cn.wustlinghang.wusthelper.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;

@Slf4j
@ApplicationScoped
public class RequestAgentService {
    private final LibraryApiService libraryApiService;

    public RequestAgentService(LibraryApiService libraryApiService) {
        this.libraryApiService = libraryApiService;
    }

    public String getBookCoverImageUrl(String isbn) throws RpcException {
        try {
            return libraryApiService.getBookCoverImageUrl(isbn);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.PUBLIC_API);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.PUBLIC_API,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String getBookDetail(String bookId) throws RpcException {
        try {
            return libraryApiService.getBookDetail(bookId);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.PUBLIC_API);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.PUBLIC_API,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String getCurrentLoan(String cookie) throws RpcException {
        try {
            return libraryApiService.getCurrentLoan(cookie);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.PERSONAL_API);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.PERSONAL_API,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String search(String keyword, int page, int pageSize) throws RpcException {
        try {
            return libraryApiService.search(keyword, page, pageSize);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.PUBLIC_API);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.PUBLIC_API,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String getLoanHistory(String cookie) throws RpcException {
        try {
            return libraryApiService.getLoanHistory(cookie);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.PERSONAL_API);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.PERSONAL_API,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String getOverdueSoon(String cookie) throws RpcException {
        try {
            return libraryApiService.getOverdueSoon(cookie);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.PERSONAL_API);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.PERSONAL_API,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    private LibraryRpcException wrapApiException(ApiException e, LibraryRpcException.SubModuleCode subModuleCode) {
        return switch (e.getCode()) {
            case NETWORK_EXCEPTION -> new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    subModuleCode,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
            case COOKIE_INVALID -> new LibraryRpcException(
                    LibraryRpcException.TypeCode.AUTH_EXCEPTION,
                    subModuleCode,
                    LibraryRpcException.ErrorCode.COOKIE_INVALID
            );
            default -> {
                log.error("本科生：{}代理请求异常，异常未处理", subModuleCode.name());
                log.error("异常：", e);
                yield new LibraryRpcException(
                        LibraryRpcException.TypeCode.AUTH_EXCEPTION,
                        subModuleCode,
                        LibraryRpcException.ErrorCode.COOKIE_INVALID
                );
            }
        };
    }
}
