package cn.wustlinghang.wusthelper.internal.physics.services;

import cn.wustlinghang.mywust.core.request.service.auth.PhysicsLogin;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.exception.ParseException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.physics.exception.PhysicsRpcException;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;

@Slf4j
@ApplicationScoped
public class LoginService {
    private final RequestClientOption option;

    private final PhysicsLogin physicsLogin;

    public LoginService(RequestClientOption option,
                        PhysicsLogin physicsLogin) {
        this.option = option;
        this.physicsLogin = physicsLogin;
    }

    public String login(String username, String password) throws PhysicsRpcException {
        try {
            return physicsLogin.getLoginCookie(username, password, option);
        } catch (ApiException e) {
            throw wrapApiException(e);
        } catch (IOException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.AUTH,
                    PhysicsRpcException.ErrorCode.NETWORK_ERROR
            );
        } catch (ParseException e) {
            log.error("物理实验：登录时出现页面参数解析问题", e);
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.AUTH,
                    PhysicsRpcException.ErrorCode.AUTH_UNKNOWN_ERROR
            );
        }
    }

    public boolean verify(String cookie) throws PhysicsRpcException {
        try {
            return physicsLogin.checkCookie(cookie, option);
        } catch (IOException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.AUTH,
                    PhysicsRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    private PhysicsRpcException wrapApiException(ApiException e) {
        PhysicsRpcException.ErrorCode errorCode = switch (e.getCode()) {
            case NETWORK_EXCEPTION -> PhysicsRpcException.ErrorCode.NETWORK_ERROR;
            case PHYSICS_PASSWORD_WRONG -> PhysicsRpcException.ErrorCode.AUTH_PASSWORD_WRONG;
            case PHYSICS_NOT_CURRENT_TERM -> PhysicsRpcException.ErrorCode.AUTH_USER_NOT_CURRENT_TERM;
            default -> {
                log.error("物理实验：登录代理请求异常，异常未处理");
                log.error("异常：", e);
                yield PhysicsRpcException.ErrorCode.AUTH_UNKNOWN_ERROR;
            }
        };

        return new PhysicsRpcException(
                PhysicsRpcException.TypeCode.AUTH_EXCEPTION,
                PhysicsRpcException.SubModuleCode.AUTH,
                errorCode
        );
    }
}
