package cn.wustlinghang.wusthelper.internal.physics.services;

import cn.wustlinghang.mywust.core.parser.physics.PhysicsCoursePageParser;
import cn.wustlinghang.mywust.core.parser.physics.PhysicsScorePageParser;
import cn.wustlinghang.mywust.data.global.Score;
import cn.wustlinghang.mywust.data.physics.PhysicsCourse;
import cn.wustlinghang.wusthelper.internal.physics.exception.PhysicsRpcException;
import cn.wustlinghang.wusthelper.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.util.ArrayList;
import java.util.List;

@ApplicationScoped
public class ParseService {
    private final PhysicsCoursePageParser coursePageParser;
    private final PhysicsScorePageParser scorePageParser;

    public ParseService(PhysicsCoursePageParser coursePageParser,
                        PhysicsScorePageParser scorePageParser) {
        this.coursePageParser = coursePageParser;
        this.scorePageParser = scorePageParser;
    }

    public List<PhysicsCourse> parseCourseTable(String data) throws PhysicsRpcException {
        try {
            return coursePageParser.parse(data);
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.COURSE_TABLE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public List<Score> parseScore(String data) throws RpcException {
        try {
            return scorePageParser.parse(data);
        } catch (PhysicsRpcException e) {
            throw e;
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.SCORE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public List<Score> parseAllScore(List<String> pages) throws RpcException {
        try {
            List<Score> scores = new ArrayList<>();
            for (String page : pages) {
                scores.addAll(scorePageParser.parse(page));
            }

            return scores;
        } catch (PhysicsRpcException e) {
            throw e;
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.SCORE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
