package cn.wustlinghang.wusthelper.internal.physics.services;

import cn.wustlinghang.mywust.core.request.service.physics.PhysicsCourseApiService;
import cn.wustlinghang.mywust.core.request.service.physics.PhysicsScoreApiService;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.exception.ParseException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.physics.exception.PhysicsRpcException;
import cn.wustlinghang.wusthelper.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.util.List;

@Slf4j
@ApplicationScoped
public class RequestAgentService {

    private final RequestClientOption requestClientOption;

    private final PhysicsCourseApiService physicsCourseApiService;
    private final PhysicsScoreApiService physicsScoreApiService;

    public RequestAgentService(RequestClientOption requestClientOption,
                               PhysicsCourseApiService physicsCourseApiService,
                               PhysicsScoreApiService physicsScoreApiService) {
        this.requestClientOption = requestClientOption;

        this.physicsCourseApiService = physicsCourseApiService;
        this.physicsScoreApiService = physicsScoreApiService;
    }

    public String getCourseTable(String cookie) throws RpcException {
        try {
            return physicsCourseApiService.getPage(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, PhysicsRpcException.SubModuleCode.COURSE_TABLE);
        } catch (IOException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.COURSE_TABLE,
                    PhysicsRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public List<String> getScore(String cookie) throws RpcException {
        try {
            return physicsScoreApiService.getAllPages(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, PhysicsRpcException.SubModuleCode.SCORE);
        } catch (IOException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.SCORE,
                    PhysicsRpcException.ErrorCode.NETWORK_ERROR
            );
        } catch (ParseException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.OTHER_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.SCORE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    private PhysicsRpcException wrapApiException(ApiException e, PhysicsRpcException.SubModuleCode subModuleCode) {
        return switch (e.getCode()) {
            case NETWORK_EXCEPTION -> new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                    subModuleCode,
                    PhysicsRpcException.ErrorCode.NETWORK_ERROR
            );
            case COOKIE_INVALID -> new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.AUTH_EXCEPTION,
                    subModuleCode,
                    PhysicsRpcException.ErrorCode.COOKIE_INVALID
            );
            default -> {
                log.error("物理实验：{}代理请求异常，异常未处理", subModuleCode.name());
                log.error("异常：", e);
                yield new PhysicsRpcException(
                        PhysicsRpcException.TypeCode.AUTH_EXCEPTION,
                        subModuleCode,
                        PhysicsRpcException.ErrorCode.COOKIE_INVALID
                );
            }
        };
    }
}
