package cn.wustlinghang.wusthelper.internal.undergrad.services;

import cn.wustlinghang.mywust.core.parser.undergraduate.*;
import cn.wustlinghang.mywust.data.global.Course;
import cn.wustlinghang.mywust.data.global.Score;
import cn.wustlinghang.mywust.data.global.StudentInfo;
import cn.wustlinghang.mywust.data.undergrad.ExamDelayApplication;
import cn.wustlinghang.mywust.exception.ParseException;
import cn.wustlinghang.wusthelper.internal.undergrad.exception.UndergradRpcException;
import cn.wustlinghang.wusthelper.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.util.List;

@ApplicationScoped
public class ParseService {
    private final UndergradCourseTableParser courseTableParser;
    private final UndergradScoreParser scoreParser;
    private final UndergradStudentInfoPageParser studentInfoPageParser;
    private final UndergradTrainingPlanPageParser trainingPlanPageParser;
    private final UndergradCreditStatusParser creditStatusParser;
    private final UndergradExamDelayParser examDelayParser;

    public ParseService(UndergradCourseTableParser courseTableParser,
                        UndergradScoreParser scoreParser,
                        UndergradStudentInfoPageParser studentInfoPageParser,
                        UndergradTrainingPlanPageParser trainingPlanPageParser,
                        UndergradCreditStatusParser creditStatusParser,
                        UndergradExamDelayParser examDelayParser) {

        this.courseTableParser = courseTableParser;
        this.scoreParser = scoreParser;
        this.studentInfoPageParser = studentInfoPageParser;
        this.trainingPlanPageParser = trainingPlanPageParser;
        this.creditStatusParser = creditStatusParser;
        this.examDelayParser = examDelayParser;
    }

    public List<Course> parseCourseTable(String data) throws UndergradRpcException {
        try {
            if (data.contains("评教")) {
                throw new UndergradRpcException(
                        UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                        UndergradRpcException.SubModuleCode.COURSE_TABLE,
                        UndergradRpcException.ErrorCode.NEED_EVALUATE
                );
            }

            return courseTableParser.parse(data);

        } catch (UndergradRpcException e) {
            throw e;
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.COURSE_TABLE,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public List<Score> parseScore(String data) throws RpcException {
        try {
            if (data.contains("评教")) {
                throw new UndergradRpcException(
                        UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                        UndergradRpcException.SubModuleCode.SCORE,
                        UndergradRpcException.ErrorCode.NEED_EVALUATE
                );
            }

            return scoreParser.parse(data);

        } catch (UndergradRpcException e) {
            throw e;
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.SCORE,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public StudentInfo parseStudentInfo(String data) throws RpcException {
        try {
            return studentInfoPageParser.parse(data);
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.STUDENT_INFO,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public String parseTrainingPlan(String data) throws RpcException {
        try {
            return trainingPlanPageParser.parse(data);
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.TRAINING_PLAN,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public String parseCreditStatus(String data) throws RpcException {
        try {
            return creditStatusParser.parse(data);
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.CREDIT_STATUS,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public List<ExamDelayApplication> parseExamDelayApplications(String data) throws RpcException {
        try {
            return examDelayParser.parse(data);
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.EXAM_DELAY_APPLICATION,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
